/*
// File:		MessageComposer.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.IO;
using Emai;
using Emai.Mime;

namespace Common
{
    enum Part : uint
    {
        kTextFlag = 1,		/*Plain text part or unicode text part*/
        kHtmlFlag = 1 << 1,	/*Html part*/
        kInlineFlag = 1 << 2,	/*Inline part*/
        kAttachmentFlag = 1 << 3	/*Attachment part*/
    };

    enum PartType : uint
    {
        kHtmlPart,
        kAlternativePart
    };

    class MessageComposer
    {
        private MessagePart mTextPart;
        private MessagePart mkHtmlPart;
	    private ArrayList mAttachments; 
        private ArrayList mInlines;
        
        public MessageComposer()
        {
            this.mAttachments = new ArrayList();
            this.mInlines = new ArrayList();
        }

	    private MessagePart CreateAlternativeMultipart()
        {
            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField("multipart/alternative", (UInt32)EnumNullOptions.NullOptions);
            CompositePartData alternativeEmaiPartData = new CompositePartData();

	        alternativeEmaiPartData.ContentID = null;
	        alternativeEmaiPartData.ContentType = contentType;
	        alternativeEmaiPartData.ContentDisposition = null;
	        alternativeEmaiPartData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
	        alternativeEmaiPartData.SubParts = null;

	        /*Create the alternative message part*/
            return new MessagePart(alternativeEmaiPartData, (UInt32)EnumNullOptions.NullOptions);
        }

	    private MessagePart CreateRelatedMultipart(UInt32 parameterValueFlag)
        {
            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField("multipart/related", (UInt32)EnumNullOptions.NullOptions);

            String parameterValuePtr = null;

            /*Determine the type of the child message part in message part's tree.*/
            switch (parameterValueFlag)
            {
                case (uint)PartType.kHtmlPart:
                    parameterValuePtr = "text/html";
                    break;

                case (uint)PartType.kAlternativePart:
                    parameterValuePtr = "multipart/alternative";
                    break;

                default:
                    break;
            }
            contentType.AddParameter("type", parameterValuePtr, (UInt32)EnumNullOptions.NullOptions);

            CompositePartData partData = new CompositePartData();
            partData.ContentID = null;
            partData.ContentType = contentType;
            partData.ContentDisposition = null;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
            partData.SubParts = null;

            /*Create the related message part*/
            return new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions);
        }

        private MessagePart CreateMixedMultipart()
        {
            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField("multipart/mixed");

            CompositePartData partData = new CompositePartData();
            partData.ContentID = null;
            partData.ContentType = contentType;
            partData.ContentDisposition = null;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
            partData.SubParts = null;

            /*Create the mixed message part*/
            return new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions);
        }

        private void BindAttachments(ref MessagePart inMessagePart, ref ArrayList inAttachments)
        {
            MessagePart[] parts = new MessagePart[inAttachments.Count];
            for (int i = 0; i < inAttachments.Count; i++)
            {
                parts[i] = (MessagePart)inAttachments[i];
            }
            inMessagePart.AddSubParts(parts, (UInt32)EnumNullOptions.NullOptions);
        }

	    public void SetText(Byte[] inPlainText, UInt32 inCharacterSet)
        {
            /*Set the properties for the new message part*/
            PlainTextPartData partData = new PlainTextPartData();
            partData.ContentID = null;
            partData.ContentType = null;
            partData.ContentDisposition = null;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;

            partData.CharacterSet = inCharacterSet;

            /*Create the text part*/
            this.mTextPart = new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions);
        }

        public String UnicodeText
        {
            set
            {
                /*Set the properties for the new message part*/
                UnicodeTextPartData partData = new UnicodeTextPartData();
                partData.ContentID = null;
                partData.ContentType = null;
                partData.ContentDisposition = null;
                partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
                partData.Text = value;
	
                /*Create the text part*/
                this.mTextPart = new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions);
            }
        }

        public String Html
        {
            set
            {
                /*Set the properties for the new message part*/
                HtmlTextPartData partData = new HtmlTextPartData();
                partData.ContentID = null;
                partData.ContentType = null;
                partData.ContentDisposition = null;
                partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;

                ASCIIEncoding encoding = new ASCIIEncoding();
                partData.Html = encoding.GetBytes(value);
	            
	            /*Create the html part*/
                this.mkHtmlPart = new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions);
            }
        }

        public void AddAttachmentFromData(Byte[] inData, String inFileName)
        {
            /*Check if the input parameters are correct*/
            if (inData.Length == 0 || inFileName.Length == 0)
                return;

            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField(null, (UInt32)EnumNullOptions.NullOptions);
            ContentField contentDisposition = new ContentField("attachment", (UInt32)EnumNullOptions.NullOptions);
            contentType.AddParameter("name", inFileName, (UInt32)EnumNullOptions.NullOptions);
            contentDisposition.AddParameter("filename", inFileName, (UInt32)EnumNullOptions.NullOptions);

            DataPartData partData = new DataPartData();

            partData.ContentID = null;
            partData.ContentType = contentType;
            partData.ContentDisposition = contentDisposition;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
            partData.Data = inData;

            /*Add the attachment part to the list*/
            mAttachments.Add(new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions));
        }

        public void AddAttachmentFromFile(String inFileName, String inFileFullPath)
        {
            if (inFileName.Length == 0 || inFileFullPath.Length == 0)
                return;

            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField(null, (UInt32)EnumNullOptions.NullOptions);
            ContentField contentDisposition = new ContentField("attachment", (UInt32)EnumNullOptions.NullOptions);
            contentType.AddParameter("name", inFileName, (UInt32)EnumNullOptions.NullOptions);
            contentDisposition.AddParameter("filename", inFileName, (UInt32)EnumNullOptions.NullOptions);

            FilePartData partData = new FilePartData();

            partData.ContentID = null;
            partData.ContentType = contentType;
            partData.ContentDisposition = contentDisposition;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
            partData.FilePath = inFileFullPath;

	        /*Add the attachment part to the list*/
            mAttachments.Add(new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions));
        }

        public void AddInlineFromData(Byte[] inData, String inFileName, String inContentId)
        {
            /*Check if the input parameters are correct*/
            if (inData.Length == 0 || inFileName.Length == 0)
                return;

            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField(null, (UInt32)EnumNullOptions.NullOptions);
            ContentField contentDisposition = new ContentField("inline", (UInt32)EnumNullOptions.NullOptions);
            contentType.AddParameter("name", inFileName, (UInt32)EnumNullOptions.NullOptions);
            contentDisposition.AddParameter("filename", inFileName, (UInt32)EnumNullOptions.NullOptions);

            DataPartData partData = new DataPartData();

            partData.ContentID = inContentId;
            partData.ContentType = contentType;
            partData.ContentDisposition = contentDisposition;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
            partData.Data = inData;

            /*Add the attachment part to the list*/
            mInlines.Add(new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions));
        }

        public void AddInlineFromFile(String inFileName, String inFileFullPath, String inContentId)
        {
            if (inFileName.Length == 0 || inFileFullPath.Length == 0)
                return;

            /*Set the properties for the new message part*/
            ContentField contentType = new ContentField(null, (UInt32)EnumNullOptions.NullOptions);
            ContentField contentDisposition = new ContentField("inline", (UInt32)EnumNullOptions.NullOptions);
            contentType.AddParameter("name", inFileName, (UInt32)EnumNullOptions.NullOptions);
            contentDisposition.AddParameter("filename", inFileName, (UInt32)EnumNullOptions.NullOptions);

            FilePartData partData = new FilePartData();

            partData.ContentID = inContentId;
            partData.ContentType = contentType;
            partData.ContentDisposition = contentDisposition;
            partData.Encoding = (UInt32)EnumMessageEncoding.EncodingAutomatic;
            partData.FilePath = inFileFullPath;

            /*Add the attachment part to the list*/
            mInlines.Add(new MessagePart(partData, (UInt32)EnumNullOptions.NullOptions));
        }

        public Message ComposeMessage(ContactList inFromContact, ContactList inSenderContact,
                                                ContactList inToContact, ContactList inCCContact,
                                                ContactList inBCContact, String inMessageSubject)
        {
            /*Determide what kind of parts should be included to the message*/
            uint flag = (this.mTextPart != null) ? (uint)Part.kTextFlag : 0;
            flag |= (this.mkHtmlPart != null) ? (uint)Part.kHtmlFlag : 0;
            flag |= (this.mInlines.Count != 0) ? (uint)Part.kInlineFlag : 0;
            flag |= (this.mAttachments.Count != 0) ? (uint)Part.kAttachmentFlag : 0;

            /*Declaration of the message*/
            Message message = null;
            /*Declaration of the complementary variables*/
            MessagePart rootMessagePart = null;
            MessagePart alternativeMessagePart = null;
            MessagePart relatedMessagePart = null;

            /*Examine all possible variants*/
            switch (flag)
            {
                case (uint)Part.kTextFlag:
                    message = new Message(this.mTextPart, (UInt32)EnumNullOptions.NullOptions);
                    break;

                case (uint)Part.kHtmlFlag:
                    message = new Message(this.mkHtmlPart, (UInt32)EnumNullOptions.NullOptions);
                    break;

                case (uint)Part.kInlineFlag:
                    rootMessagePart = CreateMixedMultipart();
                    BindAttachments(ref rootMessagePart, ref this.mInlines);
                    break;

                case (uint)Part.kAttachmentFlag:
                    rootMessagePart = CreateMixedMultipart();
                    BindAttachments(ref rootMessagePart, ref this.mAttachments);
                    break;

                case (uint)Part.kTextFlag | (uint)Part.kHtmlFlag:
                    {
                        MessagePart[] messageParts = { this.mTextPart, this.mkHtmlPart };
                        rootMessagePart = CreateAlternativeMultipart();
                        rootMessagePart.AddSubParts(messageParts, (UInt32)EnumNullOptions.NullOptions);
                        break;
                    }

                case (uint)Part.kTextFlag | (uint)Part.kInlineFlag:
                    {
                        MessagePart[] messagePart = { this.mTextPart };
                        rootMessagePart = CreateMixedMultipart();
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mInlines);
                        break;
                    }

                case (uint)Part.kTextFlag | (uint)Part.kAttachmentFlag:
                    {
                        MessagePart[] messagePart = { this.mTextPart };
                        rootMessagePart = CreateMixedMultipart();
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                case (uint)Part.kHtmlFlag | (uint)Part.kInlineFlag:
                    {
                        MessagePart[] messagePart = { this.mkHtmlPart };
                        rootMessagePart = CreateRelatedMultipart((uint)PartType.kHtmlPart);
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mInlines);
                        break;
                    }

                case (uint)Part.kHtmlFlag | (uint)Part.kAttachmentFlag:
                    {
                        MessagePart[] messagePart = { this.mkHtmlPart };
                        rootMessagePart = CreateMixedMultipart();
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                case (uint)Part.kInlineFlag | (uint)Part.kAttachmentFlag:
                    {
                        rootMessagePart = CreateMixedMultipart();
                        BindAttachments(ref rootMessagePart, ref this.mInlines);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                case (uint)Part.kTextFlag | (uint)Part.kHtmlFlag | (uint)Part.kInlineFlag:
                    {
                        MessagePart[] messageParts = { this.mTextPart, this.mkHtmlPart };
                        alternativeMessagePart = CreateAlternativeMultipart();
                        alternativeMessagePart.AddSubParts(messageParts, (UInt32)EnumNullOptions.NullOptions);
                        rootMessagePart = CreateRelatedMultipart((uint)PartType.kAlternativePart);

                        MessagePart[] messagePart = { alternativeMessagePart };
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mInlines);
                        break;
                    }

                case (uint)Part.kTextFlag | (uint)Part.kHtmlFlag | (uint)Part.kAttachmentFlag:
                    {
                        MessagePart[] messageParts = { this.mTextPart, this.mkHtmlPart };
                        alternativeMessagePart = CreateAlternativeMultipart();
                        alternativeMessagePart.AddSubParts(messageParts, (UInt32)EnumNullOptions.NullOptions);

                        rootMessagePart = CreateMixedMultipart();
                        MessagePart[] messagePart = { alternativeMessagePart };
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                case (uint)Part.kTextFlag | (uint)Part.kInlineFlag | (uint)Part.kAttachmentFlag:
                    {
                        rootMessagePart = CreateMixedMultipart();
                        MessagePart[] messagePart = { this.mTextPart };
                        rootMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mInlines);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                case (uint)Part.kHtmlFlag | (uint)Part.kInlineFlag | (uint)Part.kAttachmentFlag:
                    {
                        relatedMessagePart = CreateRelatedMultipart((uint)PartType.kHtmlPart);
                        MessagePart[] messagePart = { this.mkHtmlPart };
                        relatedMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref relatedMessagePart, ref this.mInlines);

                        rootMessagePart = CreateMixedMultipart();
                        MessagePart[] messagePart2 = { relatedMessagePart };
                        rootMessagePart.AddSubParts(messagePart2, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                case (uint)Part.kTextFlag | (uint)Part.kHtmlFlag | (uint)Part.kInlineFlag | (uint)Part.kAttachmentFlag:
                    {
                        MessagePart[] messageParts = { this.mTextPart, this.mkHtmlPart };
                        alternativeMessagePart = CreateAlternativeMultipart();
                        alternativeMessagePart.AddSubParts(messageParts, (UInt32)EnumNullOptions.NullOptions);

                        relatedMessagePart = CreateRelatedMultipart((uint)PartType.kAlternativePart);
                        MessagePart[] messagePart = { alternativeMessagePart };
                        relatedMessagePart.AddSubParts(messagePart, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref relatedMessagePart, ref this.mInlines);

                        rootMessagePart = CreateMixedMultipart();
                        MessagePart[] messagePart2 = { relatedMessagePart };
                        rootMessagePart.AddSubParts(messagePart2, (UInt32)EnumNullOptions.NullOptions);
                        BindAttachments(ref rootMessagePart, ref this.mAttachments);
                        break;
                    }

                default:
                    break;
            }

            /*Create the out message*/
            if (rootMessagePart != null)
                message = new Message(rootMessagePart, (UInt32)EnumNullOptions.NullOptions);

            /*Add the different fields to the message*/
            if (message != null)
            {
                /*Add the contact fields to the message*/
                message.SetContactList(inFromContact, (UInt32)EnumMessageFieldType.From);

                if (inSenderContact != null)
                    message.SetContactList(inSenderContact, (UInt32)EnumMessageFieldType.Sender);
                if (inToContact != null)
                    message.SetContactList(inToContact, (UInt32)EnumMessageFieldType.To);
                if (inCCContact != null)
                    message.SetContactList(inCCContact, (UInt32)EnumMessageFieldType.CC);
                if (inBCContact != null)
                    message.SetContactList(inBCContact, (UInt32)EnumMessageFieldType.BCC);

                /*Add the subject field*/
                message.SetField(null, inMessageSubject, (UInt32)EnumMessageFieldType.Subject);

                /*Override default value of the MIME-version field*/
                message.SetField(null, "1.0", (UInt32)EnumMessageFieldType.MimeVersion);
            }

            return message;
        }
    }
}
