/*
// File:		MainForm.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Emai;
using Emai.Imap;
using Emai.Mime;

namespace Imap
{
    public partial class MainForm : Form
    {
        private ImapSession mImapSession;
        private ImapResponseHandler mResponseHandler;
        private Int32 mLastRetriveMsgIndex;
        private Int32 mLastRetriveFolderIndex;
        private ListViewItem mLastRetriveMsgItem;
        private ListViewItem mLastRetriveFolderItem;
        private Color mMsgItemColor;
        private Color mFolderItemColor;

        public MainForm()
        {
            InitializeComponent();
            this.comboBoxAuthType.SelectedIndex = 0;
            this.timer.Start();
            this.mLastRetriveMsgIndex = -1;
            this.mLastRetriveFolderIndex = -1;
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void listViewMailboxes_Resize(object sender, EventArgs e)
        {
            this.columnHeaderMailboxesNames.Width = this.listViewMailboxes.ClientSize.Width;
        }

        private void listViewMessages_Resize(object sender, EventArgs e)
        {
            this.columnHeaderMsgsNames.Width = this.listViewMessages.ClientSize.Width;
        }

        private void listViewMailboxes_RetrieveVirtualItem(object sender, RetrieveVirtualItemEventArgs e)
        {
            if (this.mLastRetriveFolderIndex != e.ItemIndex)
            {
                this.mLastRetriveFolderIndex = e.ItemIndex;
                this.mFolderItemColor = Color.Black;
                if (this.mResponseHandler.GetFolderAtIndex(e.ItemIndex) != null)
                {
                    ImapFolder folder = this.mResponseHandler.GetFolderAtIndex(e.ItemIndex);
                    this.mLastRetriveFolderItem = new ListViewItem(folder.Name);
                    if ((folder.Attributes & (UInt32)EnumImapMailboxAttributes.StandardAttibutesMask) ==
                        (UInt32)EnumImapMailboxAttributes.NoselectAttribute)
                    {
                        this.mFolderItemColor = Color.Gray;
                    }
                }
            }
            e.Item = this.mLastRetriveFolderItem;
            e.Item.ForeColor = this.mFolderItemColor;
        }

        private void listViewMessages_CacheVirtualItem(object sender, CacheVirtualItemsEventArgs e)
        {
            ImapFolder folder = GetActiveMailbox();
            if (folder != null)
            {
                if (folder.Exists > 0)
                {
                    ImapSequenceSet msgSequence = new ImapSequenceSet();
                    FetchMessageAttribute[] fetchAttr = new FetchMessageAttribute[3];
                    fetchAttr[0] = new FetchMessageAttribute((UInt32)EnumImapFetchAttribute.Flags);
                    fetchAttr[1] = new FetchMessageAttribute((UInt32)EnumImapFetchAttribute.RFC822Header);
                    fetchAttr[2] = new FetchMessageAttribute((UInt32)EnumImapFetchAttribute.Uid);
                    for (Int32 i = e.StartIndex; i <= e.EndIndex; i++)
                    {
                        if (folder.GetMessageDataAtIndex((UInt32)i + 1) == null)
                        {
                            msgSequence.AddNumber((UInt32)i + 1);
                        }
                    }
                    if (msgSequence.GetCount() > 0)
                        FetchData(fetchAttr, ref msgSequence);
                }
            }
        }

        private void listViewMessages_RetrieveVirtualItem(object sender, RetrieveVirtualItemEventArgs e)
        {
            if (this.mLastRetriveMsgIndex != e.ItemIndex)
            {
                this.mLastRetriveMsgIndex = e.ItemIndex;
                this.mMsgItemColor = Color.Black;
                ImapFolder folder = GetActiveMailbox();
                if (folder != null)
                {
                    if ((folder.Exists > 0) && (e.ItemIndex <= folder.Exists))
                    {
                        UInt32 messageNumber = (UInt32)e.ItemIndex + 1;
                        if (folder.GetMessageDataAtIndex(messageNumber) != null)
                        {
                            Emai.Mime.Message message = folder.GetMessageDataAtIndex(messageNumber).Message;
                            this.mLastRetriveMsgItem = new ListViewItem(message.GetField(null, (UInt32)EnumMessageFieldType.Subject));
                            if ((folder.GetMessageDataAtIndex(messageNumber).Flags & (UInt32)EnumImapStatusResponseFlags.Deleted) != 0)
                            {
                                if (folder.IsMarkedForDeletion(messageNumber))
                                    this.mMsgItemColor = Color.Pink;
                                else
                                    this.mMsgItemColor = Color.Red;
                            }
                       }
                        else
                        {
                            this.mLastRetriveMsgItem = new ListViewItem("");
                        }
                    }
                }
            }
            e.Item = this.mLastRetriveMsgItem;
            e.Item.ForeColor = this.mMsgItemColor;
        }

        private void buttonGet_Click(object sender, EventArgs e)
        {
            try
            {
                Emai.License.Key = this.textBoxLicKey.Text;
                this.listViewMailboxes.VirtualListSize = 0;
                this.listViewMessages.VirtualListSize = 0;

                if (this.mImapSession != null)
                    this.mImapSession = null;
                Boolean bConnected = false;
                if (!Connect(ref bConnected))
                    return;

                this.mResponseHandler.ClearFolderList();

                this.mImapSession.OnOperation += new Event_OnOperationEventHandler(this.mailit_OnOperation);
                this.mImapSession.OnDataSent += new Event_OnDataEventHandler(this.mailit_OnDataSent);
                this.mImapSession.OnDataReceived += new Event_OnDataEventHandler(this.mailit_OnDataReceived);
                this.mImapSession.OnServerError += new Event_OnServerErrorEventHandler(this.mailit_OnServerError);

                this.mImapSession.ListMailboxes("", "*", (UInt32)EnumNullOptions.NullOptions);
                MailboxesNumber = (UInt32)this.mResponseHandler.FoldersCount;
                if (MailboxesNumber > 0)
                    this.listViewMailboxes.Items[0].Selected = true;
                this.toolStripProgressBarPercent.Value = 0;
                this.buttonAnother.Enabled = true;
                this.textBoxServer.Enabled = false;
                this.textBoxAccount.Enabled = false;
                this.textBoxPassword.Enabled = false;
                this.numericUpDownPort.Enabled = false;
                this.checkBoxUseSsl.Enabled = false;
                this.buttonGet.Enabled = false;
                this.comboBoxAuthType.Enabled = false;
                this.textBoxLicKey.Enabled = false;
            }
            catch (Emai.InvalidLicenseException /*exception*/)
            {
                ResetConnection();
                MessageBox.Show("Licence key is invalid. Restart application and enter valid key.", "Error");
                this.toolStripStatusLabelText.Text = "";
            }
            catch (Emai.Exception exception)
            {
                ResetConnection();
                FormatError(exception);
            }
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            try
            {
                if (this.mImapSession != null)
                {
                    this.mImapSession.Noop();
                    ImapFolder folder = GetActiveMailbox();
                    if (folder != null)
                    {
                        MailboxesNumber = (UInt32)this.mResponseHandler.FoldersCount;
                        MessagesNumber = folder.Exists;
                        
                        this.listViewMailboxes.Items[this.mResponseHandler.GetFolderIndex(folder.Name)].Selected = true;
                    }
                }
            }
            catch (Emai.Exception exception)
            {
                ResetConnection();
                FormatError(exception);
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.timer.Stop();
            try
            {
                if (this.mImapSession != null)
                    this.mImapSession.Disconnect();
            }
            catch (Emai.Exception exception)
            {
                ResetConnection();
                FormatError(exception);
            }
        }

        private void buttonExpunge_Click(object sender, EventArgs e)
        {
            try
            {
                ImapFolder folder = GetActiveMailbox();
                if (folder != null)
                {
                    /*Expunge all messages, which have been selected*/
                    if (folder.SelectedForDeletion != null)
                    {
                        if (MessageBox.Show(this, "Do you really want to delete message(s) from server?",
                            "ImapGUI", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                        {
                            ImapSequenceSet msgs = folder.SelectedForDeletion;
                            this.mImapSession.Expunge(ref msgs);
                            folder.FlushSelectedForDeletion();
                            this.listViewMailboxes.Items[this.mResponseHandler.GetFolderIndex(folder.Name)].Selected = true;
                            MessagesNumber = folder.Exists;
                            this.toolStripStatusLabelText.Text = "";
                        }
                    }
                }
            }
            catch (Emai.Exception exception)
            {
                FormatError(exception);
            }
        }

        private void ResetConnection()
        {
			if (this.mResponseHandler != null)
				this.mResponseHandler.ClearFolderList();
            this.mImapSession = null;
            MailboxesNumber = 0;
            MessagesNumber = 0;
            this.buttonAnother.Enabled = true;
            this.textBoxServer.Enabled = true;
            this.textBoxAccount.Enabled = true;
            this.textBoxPassword.Enabled = true;
            this.numericUpDownPort.Enabled = true;
            this.checkBoxUseSsl.Enabled = true;
            this.buttonGet.Enabled = true;
            this.comboBoxAuthType.Enabled = true;
            this.textBoxLicKey.Enabled = true;
            this.toolStripStatusLabelText.Text = "";
            this.textBoxServer.Focus();
        }

        private void FormatError(Emai.Exception inException)
        {
            Console.Out.WriteLine(inException.ErrorCode);
        }

        private Boolean Connect(ref Boolean outConnected)
        {
            try
            {
                outConnected = false;
                if (this.mImapSession == null)
                {
                    UInt32 options = (UInt32)EnumNullOptions.NullOptions;
                    if (this.checkBoxUseSsl.Checked)
                        options = (UInt32)EnumImapUseSSLOption.UseSSLOption;
                    this.mImapSession = new ImapSession(60, options);
                    this.mResponseHandler = new ImapResponseHandler(ref this.mImapSession);
                    this.mImapSession.Connect(this.textBoxServer.Text, (Int16)this.numericUpDownPort.Value);

                    UInt32[] authTypes = 
                    {
			            (UInt32)EnumImapAuthModes.Automatic,
			            (UInt32)EnumImapAuthModes.Login,
			            (UInt32)EnumImapAuthModes.Plain,
			            (UInt32)EnumImapAuthModes.CramMD5,
			            (UInt32)EnumImapAuthModes.NTLM,
                        (UInt32)EnumImapAuthModes.LoginCommand
                    };

                    if (this.comboBoxAuthType.SelectedIndex != -1)
                        this.mImapSession.Authenticate(this.textBoxAccount.Text, this.textBoxPassword.Text, authTypes[this.comboBoxAuthType.SelectedIndex]);
                    outConnected = true;
                    this.toolStripProgressBarPercent.Value = 0;
                }
            }
            catch (Emai.Exception exception)
            {
                ResetConnection();
                FormatError(exception);
                return false;
            }
            return true;
        }

        private Boolean FetchData(FetchMessageAttribute[] inFetchAttr,
                            ref ImapSequenceSet msgSequence)
        {
            try
            {
                FetchMessageData fetchData = new FetchMessageData(inFetchAttr);

                // Fetch data from server for message with number inMessageNumber
                this.mImapSession.Fetch(ref msgSequence, ref fetchData, (UInt32)EnumNullOptions.NullOptions);
                this.toolStripProgressBarPercent.Value = 0;
                this.toolStripStatusLabelText.Text = "";
                this.statusStrip.Refresh();
                return true;
            }
            catch (Emai.Exception exception)
            {
                //reset IMAP session object
                FormatError(exception);
                return false;
            }
        }

        private void mailit_OnDataSent(object sender, DataEventArgs e)
        {
            this.toolStripProgressBarPercent.Value = 0;
            Int32 curPercent = (100 * (Int32)e.TransferedSize) / (Int32)e.TotalSize;
            this.toolStripProgressBarPercent.Value = curPercent;
        }

        private void mailit_OnDataReceived(object sender, DataEventArgs e)
        {
            this.toolStripProgressBarPercent.Value = 0;
            Int32 curPercent = (100 * (Int32)e.TransferedSize) / (Int32)e.TotalSize;
            this.toolStripProgressBarPercent.Value = curPercent;
        }

        private void mailit_OnServerError(object sender, ServerErrorEventArgs e)
        {
            this.toolStripStatusLabelText.Text = "The server error encountered was: " + e.ServerReply;
        }

        private void mailit_OnOperation(object sender, OperationEventArgs e)
        {
            String[] statusTexts = {
                    "Connecting to server...",
                    "Successfully connected to server.",
                    "Authenticating on server...",
                    "Successfully authenticated on server.",
                    "Creating folder...",
                    "Folder successfully created.",
                    "Deleting folder...",
                    "Folder successfully deleted.",
                    "Subsribing folder...",
                    "Folder successfully subscribed.",
                    "Unsubsribing folder...",
                    "Folder successfully unsubscribed.",
                    "Renaming folder...",
                    "Folder successfully renamed.",
                    "Retrieving folder list...",
                    "Folder list successfully retrieved.",
                    "Examining folder...",
                    "Folder successfully examined.",
                    "Retrieving folder status...",
                    "Folder status successfully retrieved.",
                    "Selecting folder...",
                    "Folder successfully selected.",
                    "Closing folder...",
                    "Folder successfully closed.",
                    "Checking...",
                    "Checked.",
                    "Appending message...",
                    "Message successfully appended.",
                    "Searching messages...",
                    "Message search finished.",
                    "Fetching messages...",
                    "Messages successfully fetched.",
                    "Copying messages...",
                    "Messages successfully copied.",
                    "Storing messages data...",
                    "Messages data successfully stored.",
                    "Expunging messages...",
                    "Messages successfully expunged.",
                    "Sending NOOP.",
                    "NOOP successfully sent.",
                    "Closing connection.",
                    "Connection successfully closed."
            };
            this.toolStripStatusLabelText.Text = statusTexts[e.OperationCode];
            this.statusStrip.Refresh();
        }

        private void listViewMailboxes_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            try
            {
                if (e.IsSelected)
                {
                    Boolean bConnected = false;
                    if (!Connect(ref bConnected))
                        return;
                    String folderName = e.Item.Text;
                    ImapFolder folder = this.mResponseHandler.FindFolderByName(folderName);
                    Debug.Assert(folder != null);
                    if ((folder.Attributes & (UInt32)EnumImapMailboxAttributes.StandardAttibutesMask) != (UInt32)EnumImapMailboxAttributes.NoselectAttribute)
                    {
                        /*Choose the folder*/
                        mImapSession.SelectMailbox(folderName, (UInt32)EnumNullOptions.NullOptions);
                        MessagesNumber = folder.Exists;
                        this.toolStripStatusLabelText.Text = "";
                    }
                }
            }
            catch (Emai.Exception exception)
            {
                //reset IMAP session object
                ResetConnection();
                FormatError(exception);
            }
   
        }

        private ImapFolder GetActiveMailbox()
        {
            if ((this.mResponseHandler.FoldersCount > 0) && ((this.listViewMailboxes.FocusedItem != null)) 
                || (this.listViewMailboxes.SelectedIndices.Count > 0))
            {
                String name;
                if (this.listViewMailboxes.SelectedIndices.Count > 0)
                {
                    Int32 selectedRow = this.listViewMailboxes.SelectedIndices[0];
                    name = this.mResponseHandler.GetFolderAtIndex(selectedRow).Name;
                }
                else
                   name = this.listViewMailboxes.FocusedItem.Text;

                if (name.Length > 0)
                {
                    ImapFolder folder = this.mResponseHandler.FindFolderByName(name);
                    if (folder != null)
                        return folder;
                }
            }
            return null;
        }

        public UInt32 MailboxesNumber
        {
            get
            {
                return (UInt32)this.listViewMailboxes.VirtualListSize;
            }

            set
            {
                // BUG: http://blogs.msdn.com/cumgranosalis/archive/2006/03/18/ListViewVirtualModeBugs.aspx
                this.listViewMailboxes.VirtualListSize = 0;
                this.listViewMailboxes.View = View.List;
                this.listViewMailboxes.VirtualListSize = (Int32)value;
                this.listViewMailboxes.View = View.Details;
            }
        }

        public UInt32 MessagesNumber
        {
            get
            {
                return (UInt32)this.listViewMessages.VirtualListSize;
            }

            set
            {
                // BUG: http://blogs.msdn.com/cumgranosalis/archive/2006/03/18/ListViewVirtualModeBugs.aspx
                this.listViewMessages.VirtualListSize = 0;
                this.listViewMessages.View = View.List;
                this.listViewMessages.VirtualListSize = (Int32)value;
                this.listViewMessages.View = View.Details;
            }
        }

        private void listViewMessages_ItemActivate(object sender, EventArgs e)
        {
            try
            {
                Boolean bConnected = false;
                if (!Connect(ref bConnected))
                    return;
                ImapFolder folder = GetActiveMailbox();
                Debug.Assert(folder != null);
                if ((folder.Attributes & (UInt32)EnumImapMailboxAttributes.StandardAttibutesMask) != (UInt32)EnumImapMailboxAttributes.NoselectAttribute)
                {
                    /*Select the folder. For broken connection.*/
                    if (bConnected)
                        this.mImapSession.SelectMailbox(folder.Name, (UInt32)EnumNullOptions.NullOptions);

                    ListView.SelectedIndexCollection messages = this.listViewMessages.SelectedIndices;
                    
                    foreach (Int32 item in messages)
                    {
                        UInt32 messageNumber = (UInt32)item + 1;
                        Boolean fetchResult = true;
                        if (!folder.GetMessageDataAtIndex(messageNumber).HasFullMessageData) 
                        {
                            FetchMessageAttribute[] fetchAttr = new FetchMessageAttribute[1];
                            fetchAttr[0] = new FetchMessageAttribute((UInt32)EnumImapFetchAttribute.RFC822Text);
                            ImapSequenceSet msgSequence = new ImapSequenceSet();
                            msgSequence.AddNumber(messageNumber, (UInt32)EnumNullOptions.NullOptions);
                            fetchResult = FetchData(fetchAttr, ref msgSequence);
                        }
                        if (fetchResult)
                        {
                            MessageForm form = new MessageForm(folder.GetMessageDataAtIndex(messageNumber).Message);
                            form.Show(this);
                        }
                    }
                }
            }
            catch (Emai.Exception exception)
            {
                FormatError(exception);
            }    
        }

        private void buttonRemoveMailbox_Click(object sender, EventArgs e)
        {
            if ((this.mImapSession != null) && (this.listViewMailboxes.SelectedIndices.Count > 0))
            {
                if (MessageBox.Show(this, "Do you really want to delete the selected folder?",
                    "ImapGUI", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    try
                    {
                        ImapFolder folder = GetActiveMailbox();
                        Debug.Assert(folder != null);
                        /*Delete the folder*/
                        this.mImapSession.DeleteMailbox(folder.Name);
                        this.mResponseHandler.DeleteFolderFromFolderList(folder.Name);
                        MailboxesNumber = (UInt32)this.mResponseHandler.FoldersCount;
                        this.listViewMailboxes.Items[0].Selected = true;
                    }
                    catch (Emai.Exception exception)
                    {
                        FormatError(exception);
                    }    
                }
            }
        }

        private void buttonAddMailbox_Click(object sender, EventArgs e)
        {
            if (this.mImapSession != null)
            {
                InputForm form = new InputForm();
                form.Text = "Create mailbox";
                form.buttonAccept.Text = "Create";
                if (form.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        ImapFolder folder = this.mResponseHandler.FindFolderByName(form.FolderName);
                        if (folder == null)
                        {
                            this.mImapSession.CreateMailbox(form.FolderName);
                            this.mImapSession.ListMailboxes("", form.FolderName);
                            MailboxesNumber = (UInt32)this.mResponseHandler.FoldersCount;
                            this.listViewMailboxes.Items[this.mResponseHandler.GetFolderIndex(form.FolderName)].Selected = true;
                        }
                        else
                        {
                            MessageBox.Show(this, String.Format("Mailbox {0:S} already exist.", form.FolderName),
                                            "ImapGUI", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                    catch (Emai.Exception exception)
                    {
                        FormatError(exception);
                    }
                }
            }
        }

        private void buttonRename_Click(object sender, EventArgs e)
        {
            if (this.mImapSession != null)
            {
                InputForm form = new InputForm();
                form.Text = "Rename mailbox";
                form.buttonAccept.Text = "OK";
                if (form.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        ImapFolder folder = GetActiveMailbox();
                        Debug.Assert(folder != null);
                        if (folder.Name.CompareTo(form.FolderName) != 0)
                        {
                            this.mImapSession.RenameMailbox(folder.Name, form.FolderName);
                            this.mResponseHandler.RenameFoldersInFolderList(folder.Name, form.FolderName);
                            MailboxesNumber = (UInt32)this.mResponseHandler.FoldersCount;
                            this.listViewMailboxes.Items[this.mResponseHandler.GetFolderIndex(form.FolderName)].Selected = true;
                        }
                    }
                    catch (Emai.Exception exception)
                    {
                        FormatError(exception);
                    }
                }
            }
        }

        private void buttonDeleteMessage_Click(object sender, EventArgs e)
        {
            if ((this.mImapSession != null) && (this.listViewMessages.SelectedIndices.Count > 0))
            {
                if (MessageBox.Show(this, "Do you really want to mark selected message(s) as deleted?",
                    "ImapGUI", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    try
                    {
                        ImapFolder folder = GetActiveMailbox();
                        Debug.Assert(folder != null);
                        StoreMessageData storeData = new StoreMessageData(
                                        new Flags((UInt32)EnumImapStatusResponseFlags.Deleted),
                                        (UInt32)EnumImapStoreOptions.AddFlags);
                        ImapSequenceSet msgSequenceUid = new ImapSequenceSet();

                        foreach (Int32 item in this.listViewMessages.SelectedIndices)
                        {
                            msgSequenceUid.AddNumber(folder.GetMessageDataAtIndex((UInt32)item + 1).Uid);
                        }

                        if (msgSequenceUid.GetCount() > 0)
                        {
                            /*Select the current message*/
                            if (folder.SelectedForDeletion == null)
                                folder.SelectedForDeletion = msgSequenceUid;
                            else
                                folder.SelectedForDeletion.AddSequenceSet(ref msgSequenceUid, (UInt32)EnumNullOptions.NullOptions);

                            this.mImapSession.Store(ref msgSequenceUid, storeData, (UInt32)EnumImapUIDOptions.ImapUIDOption);
                            this.listViewMailboxes.Items[this.mResponseHandler.GetFolderIndex(folder.Name)].Selected = true;
                            this.toolStripStatusLabelText.Text = "";
                        }

                    }
                    catch (Emai.Exception exception)
                    {
                        FormatError(exception);
                    }
                }
            }
        }

        private void buttonUndelete_Click(object sender, EventArgs e)
        {
            if ((this.mImapSession != null) && (this.listViewMessages.SelectedIndices.Count > 0))
            {
                try
                {
                    ImapFolder folder = GetActiveMailbox();
                    Debug.Assert(folder != null);
                    StoreMessageData storeData = new StoreMessageData(
                                    new Flags((UInt32)EnumImapStatusResponseFlags.Deleted),
                                    (UInt32)EnumImapStoreOptions.RemoveFlags);

                    ImapSequenceSet msgSequenceUid = new ImapSequenceSet();
                    UInt32 currentItem = 0;
                    foreach (Int32 item in this.listViewMessages.SelectedIndices)
                    {
                        currentItem = (UInt32)item + 1;
                        msgSequenceUid.AddNumber(folder.GetMessageDataAtIndex(currentItem).Uid);
                        folder.GetMessageDataAtIndex(currentItem).Flags = (folder.GetMessageDataAtIndex(currentItem).Flags & ~(UInt32)EnumImapStatusResponseFlags.Deleted);
                    }

                    if (msgSequenceUid.GetCount() > 0)
                    {
                        /*Select the current message*/
                        folder.SelectedForDeletion.RemoveSequenceSet(ref msgSequenceUid, (UInt32)EnumNullOptions.NullOptions);
                        this.mImapSession.Store(ref msgSequenceUid, storeData, (UInt32)EnumImapUIDOptions.ImapUIDOption);
                        this.listViewMailboxes.Items[this.mResponseHandler.GetFolderIndex(folder.Name)].Selected = true;
                        this.toolStripStatusLabelText.Text = "";
                    }

                }
                catch (Emai.Exception exception)
                {
                    FormatError(exception);
                }
            }
        }

        private void buttonAddMessage_Click(object sender, EventArgs e)
        {
            if (this.mImapSession != null)
            {
                AddMessageForm form = new AddMessageForm();
                for (; ; )
                {
                    if (form.ShowDialog(this) == DialogResult.OK)
                    {
                        String errorMessage = "";
                        if (form.From.Length == 0)
                            errorMessage = "You must enter sender address.";
                        else if (form.To.Length == 0)
                            errorMessage = "You must enter recipient address.";
                        if (errorMessage.Length > 0)
                        {
                            MessageBox.Show(this, errorMessage, "ImapGUI", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            continue;
                        }
                        
                        try
                        {
                            ImapFolder folder = GetActiveMailbox();
                            Debug.Assert(folder != null);
                            /*Compose the message*/

                            /*Create the message composer*/
                            this.toolStripStatusLabelText.Text = "Composing message...";
                            Common.MessageComposer messageComposer = new Common.MessageComposer();

                            /*Add the message text part*/
                            messageComposer.UnicodeText = form.MessageText;

                            /*Add contact fields to the message*/
                            ContactList contactTo = new ContactList();
                            contactTo.AddContact(form.To, null, (UInt32)EnumNullOptions.NullOptions);
                            ContactList contactFrom = new ContactList();
                            contactFrom.AddContact(form.From, null, (UInt32)EnumNullOptions.NullOptions);

                            Emai.Mime.Message message = messageComposer.ComposeMessage(contactFrom, null, contactTo,
                                null, null, form.Subject);
                            this.toolStripStatusLabelText.Text = "Message successfully composed.";

                            if (message != null)
                            {
                                /*Add the message to the folder*/
                                this.mImapSession.AppendMessage(folder.Name, null, null, ref message);
                                this.mImapSession.SelectMailbox(folder.Name, (UInt32)EnumNullOptions.NullOptions);
                                MessagesNumber = folder.Exists;
                            }
                        }
                        catch (Emai.Exception exception)
                        {
                            FormatError(exception);
                        }
                    }
                    break;
                }
            }
        }

        private void buttonAnother_Click(object sender, EventArgs e)
        {
            if (this.mImapSession != null)
                this.mImapSession.Disconnect();
            ResetConnection();
        }

        private void textBoxLicKey_Enter(object sender, EventArgs e)
        {
            if (this.textBoxLicKey.Text.CompareTo("Put your license key here.") == 0)
            {
                this.textBoxLicKey.Text = "";
            }
        }

        private void textBoxLicKey_Leave(object sender, EventArgs e)
        {
            if (this.textBoxLicKey.Text.Length == 0)
            {
                this.textBoxLicKey.Text = "Put your license key here.";
            }
        }
    }
}